'use strict';

/* General Scripts */
var ISML = require('dw/template/ISML');
var guard = require('~/cartridge/scripts/guard');

/* Logger */
var logger = dw.system.Logger.getLogger('nexi.xpay.bm');

/* XPay scripts from int_nexixpay and bm_nexixpay cartridges */
var XPayUtils = require("~/../int_nexixpay/cartridge/scripts/util/XPayUtils");
var BackofficeSVC = require("~/cartridge/scripts/svc/XPayBackoffice");


/**
 * Initialize the search reports page
 * @returns
 */
function start() {

	var args = {};
	args.errorMsg = '';
	args.infoMsg = '';

	args.topUrl = dw.web.URLUtils.url('SiteNavigationBar-ShowMenuitemOverview', 'CurrentMenuItemId', request.httpParameterMap.CurrentMenuItemId);
	if(!request.httpParameterMap.CurrentMenuItemId.value) {
		args.topUrl = request.httpParameterMap.TopUrl.value;
	}
	
    args.mainMenuName = request.httpParameterMap.mainmenuname.stringValue;
	session.forms.boprofileinfo.bc_root_name.value = args.mainMenuName;
	session.forms.boprofileinfo.bc_root_link.value = args.topUrl;
	
	var logos = [];

	var sitePrefs = dw.system.Site.getCurrent().getPreferences();	
	var profileInfoStr : String = sitePrefs.getCustom()["XPAY_MERCH_PROF_INFO"];
	
	if(profileInfoStr) {
		var profileInfo = JSON.parse(profileInfoStr);
		logos = filterLogos(profileInfo)
	}
	else {
		args.infoMsg = dw.web.Resource.msg('profileinfo.noresultsfound', 'backoffice', null);
		logger.warn(args.infoMsg);
	}
	
	args.logos = logos;
	    
	return renderPage(args);
}

/**
 * filter only required profile items
 * @returns
 */
function filterLogos(profileInfo) {
		
	return profileInfo.availableMethods.filter(function(item) {
		return item.type === "CC";		
	});
}


	/**
 * Handles form actions (search and reimbursement actions)
 * @returns
 */
function handleForm() {
		
	var triggeredAction = request.triggeredFormAction;
	//logger.warn("triggeredAction: "+triggeredAction);
	if (!empty(triggeredAction)){
		if(triggeredAction.formId === 'update') {
			return getMerchProfInfo();
		}
	}
	return start();
}

/**
 * Renders the NexiPayReportSearch template with all related args
 * @param args
 * @returns
 */
function renderPage(args) {
	args = args || {};

    return ISML.renderTemplate('MerchantProfileInfo', {
        ContinueURL     : dw.web.URLUtils.https('MerchantProfileInfo-HandleForm'),
        CurrentForms    : session.forms,        
        TOP_URL         : args.topUrl,
        MAIN_MENU_NAME  : args.mainMenuName,
        logos 		    : args.logos || [],
        errorMsg 		: args.errorMsg || '',
        infoMsg 		: args.infoMsg || ''
    });
}

/**
 * Handles the call for reports list (see BackofficeSVC.callReportsList() for details on params)
 * @param args <Object>
 * @returns JSON result
 */
function getProfileInfo(args) {
	try {
		var params = { };
		var httpResult = BackofficeSVC.callMerchProfInfo(params);
		if(!httpResult.ok) {			
			throw httpResult;
		}
		var jsonResponse = JSON.parse(httpResult.object);
		logger.debug('callMerchProfInfo-jsonResponse:\n'+JSON.stringify(jsonResponse, null, '\t'));	
		
		return jsonResponse;
	} 
	catch(ex) {
		logger.error('Exception: '+JSON.stringify(ex));
		return {
			esito: 'KO',
			errore: {
				codice: ex.error,
				messaggio: ex.errorMessage,
			}
		}
	}
}


/**
 * Gets all the reports controller's logic
 * @returns
 */
function getMerchProfInfo() {
	
	var args = {};
	args.errorMsg = '';
	args.infoMsg = '';
	args.topUrl = session.forms.boprofileinfo.bc_root_link.value;
	args.mainMenuName = session.forms.boprofileinfo.bc_root_name.value;
	
	//call to Nexi for list of reports
	var result = getProfileInfo(args);
	//result = JSON.parse('{     "esito": "OK",     "idOperazione": "1144908",     "timeStamp": "1573038158042",     "mac": "98f0863efc5ca15cd3d45638d6b6ea371df5904c",     "urlLogoNexiSmall": "https://ecommerce.nexi.it/ecomm/payment/img/logoNexiSmall.png",     "urlLogoNexiLarge": "https://ecommerce.nexi.it/ecomm/payment/img/logoNexiLarge.png",     "availableMethods": [         {             "code": "MYSI",             "description": "NexiPay",             "selectedcard": "MYSI",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/logoNexiPay.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "SCT",             "description": "MyBank",             "selectedcard": "SCT",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/mybank.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "MASTERPASS",             "description": "Masterpass",             "selectedcard": "MASTERPASS",             "image": "https://masterpass.com/dyn/img/acc/global/mp_mark_ver_blk.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "PAYPAL",             "description": "PayPal",             "selectedcard": "PAYPAL",             "image": "https://www.paypalobjects.com/webstatic/en_US/i/buttons/PP_logo_h_100x26.png",             "type": "APM",             "recurring": "Y"         },         {             "code": "APPLEPAY",             "description": "Apple Pay",             "selectedcard": "APPLEPAY",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/apple-pay.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "AMAZONPAY",             "description": "Amazon Pay",             "selectedcard": "AMAZONPAY",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/amazonpay.png",             "type": "APM",             "recurring": "Y"         },         {             "code": "MAESTRO",             "description": "Maestro",             "selectedcard": "MAESTRO",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/maestro.svg",             "type": "CC",             "recurring": "Y"         },         {             "code": "MASTERCARD",             "description": "Mastercard",             "selectedcard": "MASTERCARD",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/mastercard.svg",             "type": "CC",             "recurring": "Y"         },         {             "code": "VISA",             "description": "Visa",             "selectedcard": "VISA",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/visa.svg",             "type": "CC",             "recurring": "Y"         }     ] }');
	if(result.esito === 'OK') {
		var sitePrefs = dw.system.Site.getCurrent().getPreferences();	
		//logger.warn("XPAY_MERCH_PROF_INFO: "+sitePrefs.getCustom()["XPAY_MERCH_PROF_INFO"]);
		
		dw.system.Transaction.wrap(function () {
			sitePrefs.getCustom()["XPAY_MERCH_PROF_INFO"] = JSON.stringify(result);
		});
		//logger.warn("XPAY_MERCH_PROF_INFO: "+sitePrefs.getCustom()["XPAY_MERCH_PROF_INFO"]);
		args.logos = filterLogos(result)
		
	} 
	else {
		args.logos = [];
		if(result.errore.messaggio) {
			args.errorMsg = result.errore.messaggio;			
			if(result.errore.codice) args.errorMsg += ' ('+result.errore.codice+')';
		}
		else
			args.errorMsg = dw.web.Resource.msg('profileinfo.genericerror', 'backoffice', null);

		//args.errorMsg =  result.errore.messaggio +' ('+result.errore.codice+')';
		logger.error('ERROR: '+args.errorMsg);
	}
	
	return renderPage(args);
}


exports.Start 				= guard.ensure(['https'], start);
exports.HandleForm 			= guard.ensure(['https', 'post'], handleForm);