/**
    Author: Picchiri Alessandro 
	Date: 2019.12.02
	File: PayMailSvc.js

    Code the Web Service Call
    PayMail Method
*/

'use strict';

var XPayUtils = require('~/../int_nexixpay/cartridge/scripts/util/XPayUtils');
var logger = dw.system.Logger.getLogger('nexi.xpay.int');


/** 
  * 
  * @param {String} params.importo
  * @param {String} params.timeout 
  * @param {String} params.url  
  * @param {String} params.timeStamp 
  * 
  * @returns {Result} a json response
  */
 function getLinkPayMail(params) {

    params = params || {};
    params.timeStamp = (new Date()).getTime();

    var apiKey =  XPayUtils.XPAY_API_KEY;
    var chiaveSegreta =  XPayUtils.XPAY_TERM_WEB_SECRETKEY;    
    var codiceTransazione =  params.transactionCode;
    var importo =  params.importo;    
    var timeout =  XPayUtils.XPAY_PAYMAIL_TIMEOUT;    
    var timeStamp =  params.timeStamp;

    var mac = XPayUtils.sha1(
		'apiKey=' + apiKey + 
        'codiceTransazione=' + codiceTransazione + 
        'importo=' + importo + 
        'timeStamp=' + timeStamp +
        chiaveSegreta);	
    
    // start service    
	var paymail = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {

        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
                logger.debug('PayMailSvc - request {0} :\n{1}', this.URL, JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        },
        mockCall: function(svc: HTTPService, client: HTTPClient) {
            return {
                statusCode: 200,
                statusMessage: "Success",
                text: '{"esito":"OK","idOperazione":"1159351","timeStamp":"1575301314970","mac":"62ccca3de8af8dfb569bbe5213a07ab5f9ffa734","payMailUrl":"https://int-ecommerce.nexi.it/ecomm/ecomm/DispatcherServlet?payMailId=11257&payMailKey=b8IYCZmJttTNQym2Lo2H6G6%2BI4Y%3D&"}"}'
            };
        } 
    });
    // end service

    
    var siteName  = dw.system.Site.current.ID;
    var locale   = params.orderLocale;

    var orderNumParam = new dw.web.URLParameter('orderNum', params.orderNum);
    var orderTokenParam = new dw.web.URLParameter('orderToken', params.orderToken);
        
    var urlAction: dw.web.URLAction = new dw.web.URLAction("XPayCallback-PayOutcome",siteName,locale);    
    var urlBackAction: dw.web.URLAction = new dw.web.URLAction("XPayCallback-PayCancel",siteName,locale);    
    
    var url = encodeURI(dw.web.URLUtils.https(urlAction, orderNumParam, orderTokenParam));    
    var url_back = encodeURI(dw.web.URLUtils.https(urlBackAction, orderNumParam, orderTokenParam));
        
    var parametriAggiuntivi = {
        url_back : url_back,
        TCONTAB  : XPayUtils.XPAY_TCONTAB
    }
    
    if(XPayUtils.XPAY_S2S_ENABLED) {
        var urlPostAction: dw.web.URLAction = new dw.web.URLAction("XPayCallback-Notify", siteName, locale);        
        var urlpost = encodeURI(dw.web.URLUtils.https(urlPostAction, orderNumParam, orderTokenParam));
        parametriAggiuntivi.urlpost = urlpost;
    }
    
    var args = {
        'apiKey'             :  apiKey,        
        'importo'            :  importo,        
        'timeout'            :  timeout,        
        'codiceTransazione'  :  codiceTransazione,
        'parametriAggiuntivi':  parametriAggiuntivi,
        'url'                :  url,              
        'mac'                :  mac,        
        'timeStamp'          :  timeStamp        
    };
    
    paymail.URL += "/ecomm/api/bo/richiestaPayMail";

    // call Nexi webservice
    var result : Result = paymail.call(args);
    
    return result;    
}

// funzione non utilizzata - si tratta solo di un test
function sendMail(nome,cognome,mail,payMailUrl) {
    
    var mailer: Mail = new dw.net.Mail();    
    mailer.addTo(mail);
    mailer.setFrom("xxx@yyy");
    mailer.setSubject("Example 2 Email Test");    
    mailer.setContent("Questo è il contenuto" + payMailUrl);
    mailer.send();

    logger.debug('\n\t\r Mail inviata con successo');
}


exports.getLinkPayMail = getLinkPayMail;