'use strict';

var XPayUtils = require("~/../int_nexixpay/cartridge/scripts/util/XPayUtils");
var logger = dw.system.Logger.getLogger('nexi.xpay.bm');


/**
 * @todo: metodo da eliminare, usare invece XPayUtils.getApiBaseUrl()
 * 
 * gets the base Url for XPay backoffice services
 * @returns an url string
 */
function getServiceBaseUrl() {
	return backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice",{}).URL;
}


/**
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/elencoordini.html}
 * @param {String} params.channel - All, MyBank, CartaCredito, PayPal, sofort
 * @param {Array} params.status - Autorizzato, Negato, Annullato
 * @param {String} params.periodo - gg/mm/aaaa - gg/mm/aaaa
 */
function callTransactionsList(params) {
	
	params = params || {};
	var transactionCode = params.transactionCode, 
		period = params.period, 
		channel = params.channel, 
		status = params.status;

	var timeStamp = (new Date()).getTime();
	
	var mac = XPayUtils.sha1(
		'apiKey=' +XPayUtils.XPAY_API_KEY + 
        'codiceTransazione=' + transactionCode + 
        'periodo=' + period + 
        'canale=' + channel + 
        'timeStamp=' + timeStamp + 
        XPayUtils.XPAY_TERM_WEB_SECRETKEY);
 
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('XPayBackoffice.args(callTransactionsList):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        },
       
    });
	backofficeWs.URL += "/ecomm/api/bo/reportOrdini";
	var args = {
	        'apiKey'              : XPayUtils.XPAY_API_KEY,
	        'canale'           	  : channel,
	        'timeStamp'           : timeStamp,
	        'periodo'             : period,
	        'mac' 				  : mac,
	        'stato' 			  : status,
	        'codiceTransazione'	  : transactionCode
	};
	var result : Result = backofficeWs.call(args);
	
	return result;
}

/**
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/elencoordini.html}
 * @param {String} params.transactionCode
 */
function callTransactionDetail(params) {
	
	var alias = params.alias;
	if(alias == null){alias = XPayUtils.XPAY_TERM_WEB_ALIAS}

	var terminal = XPayUtils.getTermByAlias(alias);

	params = params || {};
	var transactionCode = params.transactionCode;

	var timeStamp = (new Date()).getTime();
	
	var mac = XPayUtils.sha1(
		'apiKey=' +terminal.alias + 
        'codiceTransazione=' + transactionCode + 
        'timeStamp=' + timeStamp + 
        terminal.secretkey);
 
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('backofficeWs (callTransactionDetail):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        },
        
    });
	backofficeWs.URL += "/ecomm/api/bo/situazioneOrdine";
	var args = {
	        'apiKey'              : terminal.alias,
	        'timeStamp'           : timeStamp,
	        'mac' 				  : mac,
	        'codiceTransazione'	  : transactionCode
	};

	var result : Result = backofficeWs.call(args);

	
	
	return result;
}

/**
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/stornorimborso.html}
 * @param {String} params.transactionCode
 * @param {Number} params.amount - amount in cents
 */
function callReimburse(params) {
	
	params = params || {};

	var alias = params.alias;
	if(alias == null){alias = XPayUtils.XPAY_TERM_WEB_ALIAS}

	var terminal = XPayUtils.getTermByAlias(alias);

	var transactionCode = params.transactionCode;
	var amount = params.amount;

	var timeStamp = (new Date()).getTime();

	var mac = XPayUtils.sha1(
		'apiKey=' +terminal.alias + 
        'codiceTransazione=' + transactionCode + 
        'divisa=' + XPayUtils.XPAY_CURRENCY +
        'importo=' + params.amount +
        'timeStamp=' + timeStamp + 
        terminal.secretkey);
 
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('XPayBackoffice.args(callReimburse):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        },
        
    });
	backofficeWs.URL += "/ecomm/api/bo/storna";
	var args = {
	        'apiKey'              : terminal.alias,
	        'timeStamp'           : timeStamp,
	        'mac' 				  : mac,
	        'codiceTransazione'	  : transactionCode,
	        'importo'			  : amount,
	        'divisa'			  : XPayUtils.XPAY_CURRENCY
	};

	var result : Result = backofficeWs.call(args);
	
	return result;
}

/**
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/incasso.html}
 * @param {String} params.transactionCode
 * @param {Number} params.amount - amount in cents
 */
function callTake(params) {
	
	params = params || {};

	var alias = params.alias;
	if(alias == null){alias = XPayUtils.XPAY_TERM_WEB_ALIAS}

	var terminal = XPayUtils.getTermByAlias(alias);

	var transactionCode = params.transactionCode;
	var amount = params.amount;

	var timeStamp = (new Date()).getTime();
	
	var mac = XPayUtils.sha1(
		'apiKey=' +terminal.alias + 
        'codiceTransazione=' + transactionCode + 
        'divisa=' + XPayUtils.XPAY_CURRENCY +
        'importo=' + params.amount +
        'timeStamp=' + timeStamp + 
		terminal.secretkey);
 
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('XPayBackoffice.args(callTake):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
            return client.text;
        },
        
    });
	backofficeWs.URL += "/ecomm/api/bo/contabilizza";
	var args = {
	        'apiKey'              : terminal.alias,
	        'timeStamp'           : timeStamp,
	        'mac' 				  : mac,
	        'codiceTransazione'	  : transactionCode,
	        'importo'			  : amount,
	        'divisa'			  : XPayUtils.XPAY_CURRENCY
	};

	var result : Result = backofficeWs.call(args);
	
	return result;
}


/**
 * @link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/ReportPaymail.html
 * @param {String} params.refDate - a string formatted as gg/mm/aaaa
 */
function callPayMailReportList(params) {
	
	params = params || {};
	var fromDate = params.fromDate;
	var toDate = params.toDate;
	var timeStamp = (new Date()).getTime();
	var mac = XPayUtils.sha1('apiKey=' + XPayUtils.XPAY_API_KEY + "timeStamp=" + timeStamp + XPayUtils.XPAY_TERM_WEB_SECRETKEY);
	
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('XPayBackoffice.args(callPayMailReportList):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
			return client.text;
        },
        mockCall: function(svc: HTTPService, client: HTTPClient) {
            return {
                statusCode: 200,
                statusMessage: "Success",
                text: '{"esito":"OK","idOperazione":"1133731","timeStamp":"1570784653685","mac":"c7afadfc3e88bfc998401370d3c4ad96c2f43566","listaReport":[{"nomeFile":"00018210_ID_REPORT_422_09-10-2019.txt","titolo":"Test Report 02","dataElaborazione":"10/10/2019 23:01:50","formato":"txt","id":"15","frequenza":"giornaliera"},{"nomeFile":"00018210_ID_REPORT_421_09-10-2019.txt","titolo":"Test Report 01","dataElaborazione":"10/10/2019 23:01:50","formato":"txt","id":"22","frequenza":"giornaliera"},{"nomeFile":"00018210_ID_REPORT_441_09-10-2019.csv","titolo":"Test Report 03","dataElaborazione":"10/10/2019 23:01:50","formato":"csv","id":"23","frequenza":"settimanale"}]}'
            };
        }       
    });
	backofficeWs.URL += "/ecomm/api/bo/ReportPayMail";
	var args = {
			'apiKey'              : XPayUtils.XPAY_API_KEY,
			'timeStamp'           : timeStamp,
			'mac' 				  : mac,
			'linkCreatiDal'       : fromDate,	        
			'linkCreatiAl'        : toDate
	        
	};
	var result : Result = backofficeWs.call(args);
	
	return result;
}

/**
 * {@link https://ecommerce.nexi.it/specifiche-tecniche/apibackoffice/report/elenco.html}
 * @param {String} params.refDate - a string formatted as gg/mm/aaaa
 */
function callReportList(params) {
	
	params = params || {};
	var refDate = params.refDate;

	var timeStamp = (new Date()).getTime();
	
	var mac = XPayUtils.sha1(
		'apiKey=' +XPayUtils.XPAY_API_KEY + 
        'timeStamp=' + timeStamp + 
        XPayUtils.XPAY_TERM_WEB_SECRETKEY);
 
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('XPayBackoffice.args(callReportList):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
			return client.text;
        },
        mockCall: function(svc: HTTPService, client: HTTPClient) {
            return {
                statusCode: 200,
                statusMessage: "Success",
                text: '{"esito":"OK","idOperazione":"1133731","timeStamp":"1570784653685","mac":"c7afadfc3e88bfc998401370d3c4ad96c2f43566","listaReport":[{"nomeFile":"00018210_ID_REPORT_422_09-10-2019.txt","titolo":"Test Report 02","dataElaborazione":"10/10/2019 23:01:50","formato":"txt","id":"15","frequenza":"giornaliera"},{"nomeFile":"00018210_ID_REPORT_421_09-10-2019.txt","titolo":"Test Report 01","dataElaborazione":"10/10/2019 23:01:50","formato":"txt","id":"22","frequenza":"giornaliera"},{"nomeFile":"00018210_ID_REPORT_441_09-10-2019.csv","titolo":"Test Report 03","dataElaborazione":"10/10/2019 23:01:50","formato":"csv","id":"23","frequenza":"settimanale"}]}'
            };
        }       
    });
	backofficeWs.URL += "/ecomm/api/bo/elencoReport";
	var args = {
	        'apiKey'              : XPayUtils.XPAY_API_KEY,
	        'refDate'             : refDate,
	        'timeStamp'           : timeStamp,
	        'mac' 				  : mac
	};
	var result : Result = backofficeWs.call(args);
	
	return result;
}



/**
 * retrieve the merchant profile info from /ecomm/api/profileInfo API
  */
function callMerchProfInfo(params) {
	
	params = params || {};

	var timeStamp = (new Date()).getTime();
	
	var mac = XPayUtils.sha1(
		'apiKey=' +XPayUtils.XPAY_API_KEY + 
        'timeStamp=' + timeStamp + 
        XPayUtils.XPAY_TERM_WEB_SECRETKEY);
 
	var backofficeWs = dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice", {
        createRequest: function(svc: HTTPService, args) {
            svc.setRequestMethod("POST");
            if(args) {
            	logger.debug('XPayBackoffice.args(callGetPaymentLogos):\n'+JSON.stringify(args, null, '\t'));
            	svc.addHeader("Content-Type", "application/json");            	
            	return JSON.stringify(args);
            }            
        },
        parseResponse: function(svc: HTTPService, client: HTTPClient) {
			return client.text;
        },
        mockCall: function(svc: HTTPService, client: HTTPClient) {
            return {
                statusCode: 200,
                statusMessage: "Success",
                text: '{     "esito": "OK",     "idOperazione": "1144908",     "timeStamp": "1573038158042",     "mac": "98f0863efc5ca15cd3d45638d6b6ea371df5904c",     "urlLogoNexiSmall": "https://ecommerce.nexi.it/ecomm/payment/img/logoNexiSmall.png",     "urlLogoNexiLarge": "https://ecommerce.nexi.it/ecomm/payment/img/logoNexiLarge.png",     "availableMethods": [         {             "code": "MYSI",             "description": "NexiPay",             "selectedcard": "MYSI",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/logoNexiPay.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "SCT",             "description": "MyBank",             "selectedcard": "SCT",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/mybank.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "MASTERPASS",             "description": "Masterpass",             "selectedcard": "MASTERPASS",             "image": "https://masterpass.com/dyn/img/acc/global/mp_mark_ver_blk.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "PAYPAL",             "description": "PayPal",             "selectedcard": "PAYPAL",             "image": "https://www.paypalobjects.com/webstatic/en_US/i/buttons/PP_logo_h_100x26.png",             "type": "APM",             "recurring": "Y"         },         {             "code": "APPLEPAY",             "description": "Apple Pay",             "selectedcard": "APPLEPAY",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/apple-pay.svg",             "type": "APM",             "recurring": "N"         },         {             "code": "AMAZONPAY",             "description": "Amazon Pay",             "selectedcard": "AMAZONPAY",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/amazonpay.png",             "type": "APM",             "recurring": "Y"         },         {             "code": "MAESTRO",             "description": "Maestro",             "selectedcard": "MAESTRO",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/maestro.svg",             "type": "CC",             "recurring": "Y"         },         {             "code": "MASTERCARD",             "description": "Mastercard",             "selectedcard": "MASTERCARD",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/mastercard.svg",             "type": "CC",             "recurring": "Y"         },         {             "code": "VISA",             "description": "Visa",             "selectedcard": "VISA",             "image": "https://ecommerce.nexi.it/ecomm/payment/img/visa.svg",             "type": "CC",             "recurring": "Y"         }     ] }'
            };
        }       
    });
	backofficeWs.URL += "/ecomm/api/profileInfo";
	var args = {
	        'apiKey'              : XPayUtils.XPAY_API_KEY,
	        'timeStamp'           : timeStamp,
			'mac' 				  : mac,
			'platform'            : 'CommCloud_'+XPayUtils.XPAY_ENVIRONMENT,
			'platformVers'        : 'v'+dw.system.System.compatibilityMode,
			'pluginVers'          : '1.0'
						
	};
	var result : Result = backofficeWs.call(args);
	
	return result;
}

/**
 * Transform a yyyy-mm-dd date string into dd/mm/yyyy 
 * @param val <String> yyyy-mm-dd
 * @returns <String> dd-mm-yyyy
 */
function getNexiDateFromString(val) {
	if(!val) return null;
	if(val.length !== 10) return null;
	var tmp = val.split('-');
	return tmp[2]+'/'+tmp[1]+'/'+tmp[0];
}

exports.getNexiDateFromString 	= getNexiDateFromString;
exports.getServiceBaseUrl 	    = getServiceBaseUrl;
exports.callTransactionsList 	= callTransactionsList;
exports.callTransactionDetail 	= callTransactionDetail;
exports.callReimburse 			= callReimburse;
exports.callTake 				= callTake;
exports.callReportList 	        = callReportList;
exports.callPayMailReportList   = callPayMailReportList;
exports.callMerchProfInfo 	    = callMerchProfInfo;