/**
 * constants and utility functions for Nexi XPay cartridge
 */

 'use strict';

// Get Site Preferences
var sitePrefs = dw.system.Site.getCurrent().getPreferences();

// Nexi XPay API Key
var XPAY_API_KEY : String = sitePrefs.getCustom()["XPAY_API_KEY"];
var XPAY_PAYINST : String = "NEXI_XPAY";

// Nexi XPay payment instrument id
var XPAY_PAYINST : String = "NEXI_XPAY";

//terminal web
var XPAY_TERM_WEB_ALIAS : String = sitePrefs.getCustom()["XPAY_TERM_WEB_ALIAS"];
var XPAY_TERM_WEB_SECRETKEY : String = sitePrefs.getCustom()["XPAY_TERM_WEB_SECRETKEY"];
var XPAY_TERM_WEB_GROUP : String = sitePrefs.getCustom()["XPAY_TERM_WEB_GROUP"];

//terminal rico
var XPAY_TERM_RICO_ALIAS : String = sitePrefs.getCustom()["XPAY_TERM_RICO_ALIAS"];
var XPAY_TERM_RICO_SECRETKEY : String = sitePrefs.getCustom()["XPAY_TERM_RICO_SECRETKEY"];
var XPAY_TERM_RICO_GROUP : String = sitePrefs.getCustom()["XPAY_TERM_RICO_GROUP"];

// Nexi XPay Pay Mode
var XPAY_PAYMODE : String = sitePrefs.getCustom()["XPAY_PAYMODE"].value;
var PM_XPAY_BUILD   = 'XPayBuild';
var PM_EASY_PAYMENT = 'EasyPayment';
var PM_ONECLICK     = 'OneClick';

// Nexi XPay XBuild Client JS
var XPAY_BUILD_JS_URL_PREFIX : String = sitePrefs.getCustom()["XPAY_BUILD_JS_URL_PREFIX"];
var XPAY_BUILD_JS_URL : String = XPAY_BUILD_JS_URL_PREFIX + XPAY_API_KEY

// Currency
var XPAY_CURRENCY : String = sitePrefs.getCustom()["XPAY_CURRENCY"];

// Supported languages 
// {@link https://ecommerce.nexi.it/specifiche-tecniche/tabelleecodifiche/codificalanguageid.html}
var LANGUAGES_MAP = {
		'it': 'ITA',
		'es': 'SPA',
		'fr': 'FRA',
		'de': 'GER',
		'ja': 'JPN',
		'zh': 'CHI',
		'en': 'ENG',
		//'ar': 'ARA',
		//'ru': 'RUS',
		//'pr': 'POR'
};
var defaultLanguage : String = sitePrefs.getCustom()["XPAY_DEF_LANGUAGE"];

//Environment: (INTEG | PROD)
var XPAY_ENVIRONMENT : String = sitePrefs.getCustom()["XPAY_ENVIRONMENT"].value;

// TCONTAB: C (immediata) |  D (differita)
var XPAY_TCONTAB : String = sitePrefs.getCustom()["XPAY_TCONTAB"].value;

//Server to server switch
var XPAY_S2S_ENABLED : Boolean = sitePrefs.getCustom()["XPAY_S2S_ENABLED"];

// PayMail
var XPAY_PAYMAIL_TIMEOUT : String = sitePrefs.getCustom()["XPAY_PAYMAIL_TIMEOUT"].value;

/**
 * Creates a unique transaction id
 */
function generateUUIDCode(){
	return dw.util.UUIDUtils.createUUID();
}

/**
 * Gets the current language code 
 * @param pdict
 * @returns {String} languageId 
 */
function getLanguage(pdict){
	//var languageCode = (pdict.CurrentRequest.locale || 'en');
	var languageCode = (pdict.CurrentRequest.locale || 'default').split('_')[0];
	return LANGUAGES_MAP[languageCode] || defaultLanguage;
}

/**
 * encrypt a string with SHA-1 algorithm
 * @param {String} message an arbitrary string
 * @returns a 160bit digest string
 */
function sha1(message) {
	var sha1Encoder = new dw.crypto.MessageDigest(dw.crypto.MessageDigest.DIGEST_SHA_1);	
	return dw.crypto.Encoding.toHex(sha1Encoder.digestBytes(new dw.util.Bytes(message, 'UTF-8')));
}

/**
 * gets the base Url for XPay api
 * @returns an url string
 */
function getApiBaseUrl() {
	return dw.svc.LocalServiceRegistry.createService("Nexi.XPay.restservice",{}).URL;
}

function getTermByAlias(alias) {
	if(alias == XPAY_TERM_RICO_ALIAS) {
		return {
			alias : XPAY_TERM_RICO_ALIAS,
			secretkey : XPAY_TERM_RICO_SECRETKEY,
			group : XPAY_TERM_RICO_GROUP
		};
	}
	else {
		return {
			alias : XPAY_TERM_WEB_ALIAS,
			secretkey : XPAY_TERM_WEB_SECRETKEY,
			group : XPAY_TERM_WEB_GROUP
		};
	}
}

/**
 * dinamically check at runtime if recurrent payments feature is available
 */
function isRecurringEnabled(){
	return dw.object.SystemObjectMgr.describe('Order').getCustomAttributeDefinition('xpay_recurContractNum');
}


exports.XPAY_PAYINST = XPAY_PAYINST;
exports.XPAY_TERM_WEB_ALIAS = XPAY_TERM_WEB_ALIAS;
exports.XPAY_TERM_WEB_SECRETKEY = XPAY_TERM_WEB_SECRETKEY;
exports.XPAY_TERM_WEB_GROUP = XPAY_TERM_WEB_GROUP;
exports.XPAY_TERM_RICO_ALIAS = XPAY_TERM_RICO_ALIAS;
exports.XPAY_TERM_RICO_SECRETKEY = XPAY_TERM_RICO_SECRETKEY;
exports.XPAY_TERM_RICO_GROUP = XPAY_TERM_RICO_GROUP;

exports.XPAY_API_KEY = XPAY_API_KEY;
exports.XPAY_PAYINST = XPAY_PAYINST;

exports.XPAY_PAYMODE = XPAY_PAYMODE; 
exports.PM_EASY_PAYMENT = PM_EASY_PAYMENT; 
exports.PM_XPAY_BUILD = PM_XPAY_BUILD; 
exports.PM_ONECLICK = PM_ONECLICK;
exports.PM_XPAY_BUILD = PM_XPAY_BUILD; 

exports.XPAY_BUILD_JS_URL = XPAY_BUILD_JS_URL;
exports.XPAY_CURRENCY = XPAY_CURRENCY;
exports.XPAY_ENVIRONMENT = XPAY_ENVIRONMENT;
exports.XPAY_TCONTAB = XPAY_TCONTAB;
exports.XPAY_S2S_ENABLED = XPAY_S2S_ENABLED;
exports.XPAY_PAYMAIL_TIMEOUT = XPAY_PAYMAIL_TIMEOUT;
exports.generateUUIDCode = generateUUIDCode;
exports.getLanguage = getLanguage;
exports.sha1 = sha1; 
exports.getApiBaseUrl = getApiBaseUrl; 
exports.getTermByAlias = getTermByAlias; 
exports.isRecurringEnabled = isRecurringEnabled; 

